@@ -42,6 +42,9 @@ static struct uri uris[] = {
     { "geo:13.37,42.42;u=45.5", TRUE },
     { "geo:13.37,42.42,12.12;u=45.5", TRUE },
     { "geo:13.37,42.42,12.12;crs=wgs84;u=45.5", TRUE },
+    { "geo:13.37,42.42,12.12;crs=wgs84;u=45.5;u=10", FALSE },
+    { "geo:13.37,42.42,12.12;crs=wgs84;u=45.5;crs=wgs84", FALSE },
+    { "geo:13.37,42.42,12.12;crs=wgs84;u=45.5;z=18", TRUE },
     { "geo:0.0,0,0", TRUE },
     { "geo :0.0,0,0", FALSE },
     { "geo:0.0 ,0,0", FALSE },
