@@ -405,6 +405,42 @@ test_xep (void)
 	g_list_free (res);
 }
 
+static void
+test_pi (void)
+{
+	g_autoptr (GHashTable) params = NULL;
+	GeocodeForward *object;
+	GError *error = NULL;
+	GList *res;
+	GeocodePlace *place;
+
+	/* The query parameters the mock server expects to receive. */
+	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+	add_attr_string (params, "q", "Jack Cole Building, St Andrews");
+	add_attr_string (params, "limit", "1");
+	add_attr_string (params, "bounded", "1");
+
+	object = create_forward_for_string ("Jack Cole Building, St Andrews",
+	                                    params, "pi.json");
+	geocode_forward_set_bounded (object, TRUE);
+
+	geocode_forward_set_answer_count (object, 1);
+	res = geocode_forward_search (object, &error);
+	if (res == NULL) {
+		g_warning ("Failed at geocoding: %s", error->message);
+		g_error_free (error);
+	}
+	g_assert (res != NULL);
+
+	g_object_unref (object);
+
+	g_assert_cmpint (g_list_length (res), ==, 1);
+	place = res->data;
+	g_assert_cmpstr (geocode_place_get_street_address (place), ==, "North Haugh π");
+	g_object_unref (place);
+	g_list_free (res);
+}
+
 static void
 test_pub (void)
 {
@@ -419,10 +455,11 @@ test_pub (void)
 	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	add_attr_string (params, "q", "9, old palace road, guildford, surrey");
 	add_attr_string (params, "limit", "1");
-	add_attr_string (params, "bounded", "0");
+	add_attr_string (params, "bounded", "1");
 
 	object = create_forward_for_string ("9, old palace road, guildford, surrey",
 	                                    params, "pub.json");
+	geocode_forward_set_bounded (object, TRUE);
 
 	geocode_forward_set_answer_count (object, 1);
 	res = geocode_forward_search (object, &error);
@@ -454,6 +491,10 @@ test_search (void)
 	GError *error = NULL;
 	GList *results;
 	char *old_locale;
+	GList *l;
+	gboolean got_france, got_texas;
+	got_france = FALSE;
+	got_texas = FALSE;
 
 	old_locale = g_strdup (setlocale(LC_MESSAGES, NULL));
 	setlocale (LC_MESSAGES, "en_GB.UTF-8");
@@ -476,16 +517,9 @@ test_search (void)
 	g_object_unref (forward);
 
 	g_assert_cmpint (g_list_length (results), ==, 10);
+	g_clear_pointer (&params, g_hash_table_destroy);
 
 	/* We need to find Paris in France and in Texas */
-        /* FIXME: Uncomment following and move variable declarations to top of
-         *        this function when this bug is resolved:
-         *        https://trac.openstreetmap.org/ticket/5111
-         */
-	/*GLis *l;
-        gboolean got_france, got_texas;
-	got_france = FALSE;
-	got_texas = FALSE;
 	for (l = results; l != NULL; l = l->next) {
 		GeocodeLocation *loc;
 		GeocodePlace *place = l->data;
@@ -493,10 +527,15 @@ test_search (void)
 		loc = geocode_place_get_location (place);
 		g_assert (loc != NULL);
 
+		g_message ("1. %s", geocode_place_get_state (place));
+		g_message ("2. %s", geocode_place_get_name (place));
+		g_message ("3. %s", geocode_place_get_country (place));
+		g_message ("4. %s", geocode_location_get_description (loc));
+
 		if (g_strcmp0 (geocode_place_get_state (place), "Ile-de-France") == 0 &&
-		    g_strcmp0 (geocode_place_get_name (place), "Paris") == 0 &&
+		    g_strcmp0 (geocode_place_get_name (place), "Paris, France") == 0 &&
                     g_strcmp0 (geocode_place_get_country (place), "France") == 0 &&
-		    g_strcmp0 (geocode_location_get_description (loc), "Paris") == 0)
+		    g_strcmp0 (geocode_location_get_description (loc), "Paris, France") == 0)
 			got_france = TRUE;
 		else if (g_strcmp0 (geocode_place_get_state (place), "Texas") == 0 &&
 			 g_strcmp0 (geocode_place_get_country (place), "United States of America") == 0 &&
@@ -513,12 +552,10 @@ test_search (void)
 	g_list_free (results);
 
 	g_assert (got_france);
-	g_assert (got_texas);*/
+	g_assert (got_texas);
 
 	setlocale (LC_MESSAGES, old_locale);
 	g_free (old_locale);
-
-	g_list_free_full (results, (GDestroyNotify) g_object_unref);
 }
 
 static void
@@ -694,6 +731,8 @@ test_locale_name (void)
 	GeocodeLocation *loc;
 	char *old_locale;
 	g_autoptr (GHashTable) params = NULL;
+	GList *l;
+	gboolean found = FALSE;
 
 	old_locale = g_strdup (setlocale(LC_MESSAGES, NULL));
 
@@ -730,16 +769,17 @@ test_locale_name (void)
 	print_place (place);
 
 	g_list_free_full (res, (GDestroyNotify) g_object_unref);
+	g_clear_pointer (&params, g_hash_table_destroy);
 
 	/* Check Bonneville's region in French */
-        /* FIXME: Uncomment following and move variable declarations to top of
-         *        this function when this bug is resolved:
-         *        https://trac.openstreetmap.org/ticket/5111
-         */
-	/*GList *l;
-        gboolean found = FALSE;
 	setlocale (LC_MESSAGES, "fr_FR.UTF-8");
-	object = geocode_forward_new_for_string ("bonneville");
+	/* The query parameters the mock server expects to receive. */
+	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+	add_attr_string (params, "q", "bonneville");
+	add_attr_string (params, "limit", "10");
+	add_attr_string (params, "bounded", "0");
+	object = create_forward_for_string ("bonneville", params,
+					    "locale_name2.json");
 	res = geocode_forward_search (object, &error);
 	if (res == NULL) {
 		g_warning ("Failed at geocoding: %s", error->message);
@@ -754,18 +794,18 @@ test_locale_name (void)
 		loc = geocode_place_get_location (place);
 		g_assert (loc != NULL);
 
-		if (g_strcmp0 (geocode_place_get_name (place), "Bonneville, Rhône-Alpes, France") == 0 &&
-		    g_strcmp0 (geocode_place_get_state (place), "Rhône-Alpes") == 0 &&
+		if (g_strcmp0 (geocode_place_get_name (place), "Bonneville, Haute-Savoie, Auvergne-Rhône-Alpes, France") == 0 &&
+		    g_strcmp0 (geocode_place_get_state (place), "Auvergne-Rhône-Alpes") == 0 &&
 		    g_strcmp0 (geocode_place_get_country (place), "France") == 0 &&
 		    g_strcmp0 (geocode_location_get_description (loc),
-                               "Bonneville, Rhône-Alpes, France") == 0) {
+                               "Bonneville, Haute-Savoie, Auvergne-Rhône-Alpes, France") == 0) {
 		        found = TRUE;
                         break;
                 }
 	}
 
 	g_list_free_full (res, (GDestroyNotify) g_object_unref);
-	g_assert (found);*/
+	g_assert (found);
 
 	/* And reset the locale */
 	setlocale (LC_MESSAGES, old_locale);
@@ -924,6 +964,7 @@ int main (int argc, char **argv)
 		g_test_add_func ("/geocode/reverse", test_rev);
 		g_test_add_func ("/geocode/reverse_fail", test_rev_fail);
 		g_test_add_func ("/geocode/pub", test_pub);
+		g_test_add_func ("/geocode/pi", test_pi);
 		g_test_add_func ("/geocode/xep-0080", test_xep);
 		g_test_add_func ("/geocode/locale_name", test_locale_name);
 		g_test_add_func ("/geocode/locale_format", test_locale_format);
